/*
 *  Eleptech Router DXR6058 board support
 *
 *  Copyright (C) 2014 Gabor Juhos <juhosg@openwrt.org>
 *  Copyright (C) 2014 Imre Kaloz <kaloz@openwrt.org>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 2 as published
 *  by the Free Software Foundation.
 */

#include <linux/gpio.h>
#include <linux/platform_device.h>
#include <linux/ar8216_platform.h>

#include <asm/mach-ath79/ar71xx_regs.h>

#include "common.h"
#include "pci.h"
#include "dev-gpio-buttons.h"
#include "dev-eth.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-usb.h"
#include "dev-wmac.h"
#include "machtypes.h"
#include "dev-ap9x-pci.h"

#define DXR6058_GPIO_LED_USB1			4
#define DXR6058_GPIO_LED_USB2			15
#define DXR6058_GPIO_LED_STATUS			14
#define DXR6058_GPIO_LED_WPS			20
#define DXR6058_GPIO_LED_WLAN_2G		13

#define DXR6058_V3_GPIO_LED_USB2		4
#define DXR6058_V3_GPIO_LED_2G_AMBER	14
#define DXR6058_V3_GPIO_LED_5G_AMBER	15
#define DXR6058_V3_GPIO_LED_WPS_BLUE	19
#define DXR6058_V3_GPIO_LED_STATUS_BLUE	20

#define DXR6058_GPIO_BTN_RESET			17

#define DXR6058_V3_GPIO_BTN_WPS			16
#define DXR6058_V3_GPIO_BTN_RESET		17
#define DXR6058_V3_GPIO_USB2_POWER		21

#define DXR6058_KEYS_POLL_INTERVAL		20	/* msecs */
#define DXR6058_KEYS_DEBOUNCE_INTERVAL	(3 * DXR6058_KEYS_POLL_INTERVAL)

#define DXR6058_WAN_MAC_OFFSET			0
#define DXR6058_LAN_MAC_OFFSET			6
#define DXR6058_WMAC_CALDATA_OFFSET		0x1000
#define DXR6058_PCIE_CALDATA_OFFSET		0x5000

static struct gpio_led dxr6058_leds_gpio[] __initdata = {
	{
		.name		= "dxr6058:green:usb1",
		.gpio		= DXR6058_GPIO_LED_USB1,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:green:usb2",
		.gpio		= DXR6058_GPIO_LED_USB2,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:green:status",
		.gpio		= DXR6058_GPIO_LED_STATUS,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:green:wps",
		.gpio		= DXR6058_GPIO_LED_WPS,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:green:wlan2g",
		.gpio		= DXR6058_GPIO_LED_WLAN_2G,
		.active_low	= 1,
	},
};

static struct gpio_led dxr6058_v3_leds_gpio[] __initdata = {
/*	{
		.name		= "dxr6058:green:usb1",
		.gpio		= DXR6058_V3_GPIO_LED_USB1,
		.active_low	= 1,
	},*/
	{
		.name		= "dxr6058:green:usb2",
		.gpio		= DXR6058_V3_GPIO_LED_USB2,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:amber:wlan2g",
		.gpio		= DXR6058_V3_GPIO_LED_2G_AMBER,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:amber:wlan5g",
		.gpio		= DXR6058_V3_GPIO_LED_5G_AMBER,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:blue:wps",
		.gpio		= DXR6058_V3_GPIO_LED_WPS_BLUE,
		.active_low	= 1,
	},
	{
		.name		= "dxr6058:blue:status",
		.gpio		= DXR6058_V3_GPIO_LED_STATUS_BLUE,
		.active_low	= 1,
	},
};

static struct gpio_keys_button dxr6058_gpio_keys[] __initdata = {
	{
		.desc		= "Reset button",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = DXR6058_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= DXR6058_GPIO_BTN_RESET,
		.active_low	= 1,
	},
};

static struct gpio_keys_button dxr6058_v3_gpio_keys[] __initdata = {
	{
		.desc		= "Reset button",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = DXR6058_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= DXR6058_V3_GPIO_BTN_RESET,
		.active_low	= 1,
	},
	{
		.desc		= "WPS button",
		.type		= EV_KEY,
		.code		= KEY_WPS_BUTTON,
		.debounce_interval = DXR6058_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= DXR6058_V3_GPIO_BTN_WPS,
		.active_low	= 1,
	},
};

static struct ar8327_pad_cfg dxr6058_ar8327_pad0_cfg;
static struct ar8327_pad_cfg dxr6058_ar8327_pad6_cfg;
static struct ar8327_led_cfg dxr6058_ar8327_led_cfg;

static const struct ar8327_led_info dxr6058_leds_ar8327[] __initconst = {
	AR8327_LED_INFO(PHY0_0, HW, "dxr6058:green:wan"),
	AR8327_LED_INFO(PHY1_0, HW, "dxr6058:green:lan1"),
	AR8327_LED_INFO(PHY2_0, HW, "dxr6058:green:lan2"),
	AR8327_LED_INFO(PHY3_0, HW, "dxr6058:green:lan3"),
	AR8327_LED_INFO(PHY4_0, HW, "dxr6058:green:lan4"),
};

static struct ar8327_platform_data dxr6058_ar8327_data = {
	.pad0_cfg = &dxr6058_ar8327_pad0_cfg,
	.pad6_cfg = &dxr6058_ar8327_pad6_cfg,
	.port0_cfg = {
		.force_link = 1,
		.speed = AR8327_PORT_SPEED_1000,
		.duplex = 1,
		.txpause = 1,
		.rxpause = 1,
	},
	.port6_cfg = {
		.force_link = 1,
		.speed = AR8327_PORT_SPEED_1000,
		.duplex = 1,
		.txpause = 1,
		.rxpause = 1,
	},
//	.led_cfg = &dxr6058_ar8327_led_cfg,
//	.num_leds = ARRAY_SIZE(dxr6058_leds_ar8327),
//	.leds = dxr6058_leds_ar8327,
};

static struct mdio_board_info dxr6058_mdio0_info[] = {
	{
		.bus_id = "ag71xx-mdio.0",
		.phy_addr = 0,
		.platform_data = &dxr6058_ar8327_data,
	},
};

static void __init dxr6058_v1_setup(void)
{
	u8 *art = (u8 *) KSEG1ADDR(0x1fff0000);

	ath79_register_m25p80(NULL);
	ath79_register_leds_gpio(-1, ARRAY_SIZE(dxr6058_leds_gpio), dxr6058_leds_gpio);
	ath79_register_gpio_keys_polled(-1, DXR6058_KEYS_POLL_INTERVAL, ARRAY_SIZE(dxr6058_gpio_keys), dxr6058_gpio_keys);

	ath79_register_wmac(art + DXR6058_WMAC_CALDATA_OFFSET, NULL);
	
	/* Use the RGMII interface for the GMAC0 of the AR8337 switch */
	dxr6058_ar8327_pad0_cfg.mode = AR8327_PAD_MAC_RGMII;
	dxr6058_ar8327_pad0_cfg.txclk_delay_en = true;
	dxr6058_ar8327_pad0_cfg.rxclk_delay_en = true;
	dxr6058_ar8327_pad0_cfg.txclk_delay_sel = AR8327_CLK_DELAY_SEL1;
	dxr6058_ar8327_pad0_cfg.rxclk_delay_sel = AR8327_CLK_DELAY_SEL2;
	
	/* Use the SGMII interface for the GMAC6 of the AR8337 switch */
	dxr6058_ar8327_pad6_cfg.mode = AR8327_PAD_MAC_SGMII;
	dxr6058_ar8327_pad6_cfg.rxclk_delay_en = true;
	dxr6058_ar8327_pad6_cfg.rxclk_delay_sel = AR8327_CLK_DELAY_SEL0;
	
	dxr6058_ar8327_led_cfg.led_ctrl0 = 0xc737c737;
	dxr6058_ar8327_led_cfg.led_ctrl1 = 0x00000000;
	dxr6058_ar8327_led_cfg.led_ctrl2 = 0x00000000;
	dxr6058_ar8327_led_cfg.led_ctrl3 = 0x0030c300;
	dxr6058_ar8327_led_cfg.open_drain = true;
	dxr6058_ar8327_data.led_cfg = &dxr6058_ar8327_led_cfg,
	dxr6058_ar8327_data.num_leds = ARRAY_SIZE(dxr6058_leds_ar8327),
	dxr6058_ar8327_data.leds = dxr6058_leds_ar8327,
	
	ath79_register_mdio(0, 0x0);
	mdiobus_register_board_info(dxr6058_mdio0_info, ARRAY_SIZE(dxr6058_mdio0_info));

	ath79_setup_qca955x_eth_cfg(QCA955X_ETH_CFG_RGMII_EN);
	ath79_init_mac(ath79_eth0_data.mac_addr, art + DXR6058_WAN_MAC_OFFSET, 0);
	ath79_init_mac(ath79_eth1_data.mac_addr, art + DXR6058_LAN_MAC_OFFSET, 0);

	ath79_eth0_pll_data.pll_1000 = 0xa6000000;
	ath79_eth1_pll_data.pll_1000 = 0x03000101;

	/* GMAC0 is connected to the RMGII interface */
	ath79_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_RGMII;
	ath79_eth0_data.phy_mask = BIT(0);
	ath79_eth0_data.mii_bus_dev = &ath79_mdio0_device.dev;
	ath79_register_eth(0);

	/* GMAC1 is connected to the SGMII interface */
	ath79_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_SGMII;
	ath79_eth1_data.speed = SPEED_1000;
	ath79_eth1_data.duplex = DUPLEX_FULL;
	ath79_register_eth(1);

	ath79_register_pci();
	ath79_register_usb();
}

static void __init dxr6058_v2_setup(void)
{
	u8 *art = (u8 *) KSEG1ADDR(0x1f060000);
	u8 wlan_24g_mac[6];
	
	ath79_register_m25p80(NULL);
	ath79_register_leds_gpio(-1, ARRAY_SIZE(dxr6058_leds_gpio), dxr6058_leds_gpio);
	ath79_register_gpio_keys_polled(-1, DXR6058_KEYS_POLL_INTERVAL, ARRAY_SIZE(dxr6058_gpio_keys), dxr6058_gpio_keys);

//	ath79_register_wmac(art + DXR6058_WMAC_CALDATA_OFFSET, NULL);
	
	/* GMAC0 of the AR8327 switch is connected to the QCA9558 SoC via SGMII */
	dxr6058_ar8327_pad0_cfg.mode = AR8327_PAD_MAC_SGMII;
	dxr6058_ar8327_pad0_cfg.sgmii_delay_en = true;
	
	/* GMAC6 of the AR8327 switch is connected to the QCA9558 SoC via RGMII */
	dxr6058_ar8327_pad6_cfg.mode = AR8327_PAD_MAC_RGMII;
	dxr6058_ar8327_pad6_cfg.txclk_delay_en = true;
	dxr6058_ar8327_pad6_cfg.rxclk_delay_en = true;
	dxr6058_ar8327_pad6_cfg.txclk_delay_sel = AR8327_CLK_DELAY_SEL1;
	dxr6058_ar8327_pad6_cfg.rxclk_delay_sel = AR8327_CLK_DELAY_SEL2;
	
	dxr6058_ar8327_led_cfg.led_ctrl0 = 0xc737c737;
	dxr6058_ar8327_led_cfg.led_ctrl1 = 0x00000000;
	dxr6058_ar8327_led_cfg.led_ctrl2 = 0x00000000;
	dxr6058_ar8327_led_cfg.led_ctrl3 = 0x0030c300;
	dxr6058_ar8327_led_cfg.open_drain = true;
	dxr6058_ar8327_data.led_cfg = &dxr6058_ar8327_led_cfg,
	dxr6058_ar8327_data.num_leds = ARRAY_SIZE(dxr6058_leds_ar8327),
	dxr6058_ar8327_data.leds = dxr6058_leds_ar8327,
	
	ath79_register_mdio(0, 0x0);
	mdiobus_register_board_info(dxr6058_mdio0_info, ARRAY_SIZE(dxr6058_mdio0_info));

	ath79_setup_qca955x_eth_cfg(QCA955X_ETH_CFG_RGMII_EN);
	ath79_init_mac(ath79_eth0_data.mac_addr, art + DXR6058_WAN_MAC_OFFSET, 0);
	ath79_init_mac(ath79_eth1_data.mac_addr, art + DXR6058_WAN_MAC_OFFSET, 1);
	ath79_init_mac(wlan_24g_mac, art + DXR6058_WAN_MAC_OFFSET, 2);
	printk("%x:%x:%x:%x:%x:%x\n", wlan_24g_mac[0], wlan_24g_mac[1], wlan_24g_mac[2], 
				wlan_24g_mac[3],wlan_24g_mac[4], wlan_24g_mac[5]);
	ath79_register_wmac(art + DXR6058_WMAC_CALDATA_OFFSET, wlan_24g_mac);

	ath79_eth0_pll_data.pll_1000 = 0x56000000;
	ath79_eth1_pll_data.pll_1000 = 0x03000101;

	/* GMAC0 is connected to the RMGII interface */
	ath79_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_RGMII;
	ath79_eth0_data.phy_mask = BIT(0);
	ath79_eth0_data.mii_bus_dev = &ath79_mdio0_device.dev;
	ath79_register_eth(0);

	/* GMAC1 is connected to the SGMII interface */
	ath79_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_SGMII;
	ath79_eth1_data.speed = SPEED_1000;
	ath79_eth1_data.duplex = DUPLEX_FULL;
	ath79_register_eth(1);

	ath79_register_pci();
	ath79_register_usb();
}

static void __init dxr6058_v3_setup(void)
{
	u8 *art = (u8 *) KSEG1ADDR(0x1f060000);
	u8 wlan_24g_mac[6];

	ath79_register_m25p80(NULL);
	ath79_register_leds_gpio(-1, ARRAY_SIZE(dxr6058_v3_leds_gpio), dxr6058_v3_leds_gpio);
	ath79_register_gpio_keys_polled(-1, DXR6058_KEYS_POLL_INTERVAL, ARRAY_SIZE(dxr6058_v3_gpio_keys), dxr6058_v3_gpio_keys);
	
//	ath79_register_wmac(art + DXR6058_WMAC_CALDATA_OFFSET, NULL);
	
	/* Use the RGMII interface for the GMAC0 of the AR8337 switch */
	dxr6058_ar8327_pad0_cfg.mode = AR8327_PAD_MAC_RGMII;
	dxr6058_ar8327_pad0_cfg.txclk_delay_en = true;
	dxr6058_ar8327_pad0_cfg.rxclk_delay_en = true;
	dxr6058_ar8327_pad0_cfg.txclk_delay_sel = AR8327_CLK_DELAY_SEL1;
	dxr6058_ar8327_pad0_cfg.rxclk_delay_sel = AR8327_CLK_DELAY_SEL2;
	
	/* Use the SGMII interface for the GMAC6 of the AR8337 switch */
	dxr6058_ar8327_pad6_cfg.mode = AR8327_PAD_MAC_SGMII;
	dxr6058_ar8327_pad6_cfg.rxclk_delay_en = true;
	dxr6058_ar8327_pad6_cfg.rxclk_delay_sel = AR8327_CLK_DELAY_SEL0;
	
	ath79_register_mdio(0, 0x0);
	mdiobus_register_board_info(dxr6058_mdio0_info, ARRAY_SIZE(dxr6058_mdio0_info));
	
	ath79_setup_qca955x_eth_cfg(QCA955X_ETH_CFG_RGMII_EN);
	ath79_init_mac(ath79_eth0_data.mac_addr, art + DXR6058_WAN_MAC_OFFSET, 0);
	ath79_init_mac(ath79_eth1_data.mac_addr, art + DXR6058_WAN_MAC_OFFSET, 1);
	ath79_init_mac(wlan_24g_mac, art + DXR6058_WAN_MAC_OFFSET, 2);
	ath79_register_wmac(art + DXR6058_WMAC_CALDATA_OFFSET, wlan_24g_mac);
	
	ath79_eth0_pll_data.pll_1000 = 0xa6000000;
	ath79_eth1_pll_data.pll_1000 = 0x03000101;
	
	/* GMAC0 is connected to the RMGII interface */
	ath79_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_RGMII;
	ath79_eth0_data.phy_mask = BIT(0);
	ath79_eth0_data.mii_bus_dev = &ath79_mdio0_device.dev;
	ath79_register_eth(0);

	/* GMAC1 is connected to the SGMII interface */
	ath79_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_SGMII;
	ath79_eth1_data.speed = SPEED_1000;
	ath79_eth1_data.duplex = DUPLEX_FULL;
	ath79_register_eth(1);

	ath79_register_pci();
	ath79_register_usb();
	gpio_request_one(DXR6058_V3_GPIO_USB2_POWER, GPIOF_OUT_INIT_HIGH | GPIOF_EXPORT_DIR_FIXED, "USB2 power");
}

MIPS_MACHINE(ATH79_MACH_DXR6058V1, "DXR6058V1", "Eleptech Router DXR6058V1", dxr6058_v1_setup);
MIPS_MACHINE(ATH79_MACH_DXR6058V2, "DXR6058V2", "Eleptech Router DXR6058V2", dxr6058_v2_setup);
MIPS_MACHINE(ATH79_MACH_DXR6058V3, "DXR6058V3", "Eleptech Router DXR6058V3", dxr6058_v3_setup);
